﻿using System;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Reflection;

namespace Data
{
    public class NotificationObject : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void RaisePropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        protected void RaisePropertyChanged<T>(Expression<Func<T>> propertyExpression)
        {
            var propertyName = PropertySupport.ExtractPropertyName(propertyExpression);
            RaisePropertyChanged(propertyName);
        }

        public class PropertySupport
        {
            /// <summary>
            ///     プロパティを取得する式木から、プロパティ名を取得します。
            /// </summary>
            /// <typeparam name="T">名前を取得するプロパティの型</typeparam>
            /// <param name="propertyExpression">プロパティを取得する式木</param>
            /// <returns>プロパティの名前</returns>
            public static string ExtractPropertyName<T>(Expression<Func<T>> propertyExpression)
            {
                if (propertyExpression == null)
                {
                    throw new ArgumentNullException("propertyExpression");
                }

                var memberExpression = propertyExpression.Body as MemberExpression;
                if (memberExpression == null)
                {
                    throw new ArgumentException("", "propertyExpression");
                }

                var property = memberExpression.Member as PropertyInfo;
                if (property == null)
                {
                    throw new ArgumentException("", "propertyExpression");
                }

                var getMethod = property.GetGetMethod(true);
                if (getMethod.IsStatic)
                {
                    throw new ArgumentException("", "propertyExpression");
                }

                return memberExpression.Member.Name;
            }
        }
    }
}